@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-#   XenMan   -  Copyright (c) 2006 Jd & Hap Hazard
+#   XenMan   -  Copyright (c) 2007 Jd & Hap Hazard
 #   ======
 #
 # XenMan is a Xen management tool with a GTK based graphical interface
@@ -38,7 +38,7 @@ import sys,os,re,types
 
 from ManagedNode import ManagedNode
 from NodeProxy import Node
-from utils import XMConfig, constants, search_tree
+from utils import XMConfig, constants, search_tree, Poller
 from XenDomain import *
 
 
@@ -67,7 +67,10 @@ class XenNode(ManagedNode):
         self._managed_domfiles = None
         self.xenauto = '/etc/xen/auto'
 
-
+        self.metrics_helper = MetricsHelper(self)
+        self.metrics_poller = None
+        self.POLLING_INTERVAL = 5.0 # default no.of seconds between each metric fetch
+        self.MAX_POLLS = 4 # default number of times metrics poller iterates
         
 
     def _init_node_info(self):
@@ -190,6 +193,24 @@ class XenNode(ManagedNode):
     def remove_dom_config(self, filename):
         return self.dom_list.remove_dom_config(filename)
 
+    # Metrics
+    def get_metrics(self, refresh=False):
+        if refresh:
+            self.metrics = self.metrics_helper.getFrame()
+        else:
+            if self.metrics is None:
+                # first time get_metrics has been called
+                self.get_metrics(refresh=True)
+
+            if self.metrics_poller is None or not self.metrics_poller.isAlive():
+                # kick off ascynchronous metrics polling
+                # ... MAX_POLLS polls at POLLING_INTERVAL second intervals
+                self.metrics_poller = Poller(self.POLLING_INTERVAL,self.get_metrics,
+                                             args=[True],max_polls=self.MAX_POLLS)
+                self.metrics_poller.start()
+                
+        return self.metrics
+
 
     # dom operations
     
@@ -390,6 +411,83 @@ class DomListHelper:
 
 
 
+class MetricsHelper:
+    """A Helper to fetch and format runtime metrics from a Xen Host"""
+
+    FRAME_CMD = 'xentop -b -i 2 -d 1'
+    
+    def __init__(self, node):
+        self.node = node
+
+    def getFrame(self):
+        """returns a dictionary containing metrics for all running domains
+        in a frame"""
+        (retbuf, retcode) = self.node.node_proxy.exec_cmd(self.FRAME_CMD,
+                                                          self.node.exec_path)
+        if retcode: return None
+
+        # hack to eliminate unwanted vbd entries.
+        cleansed_retbuf = re.sub('vbd.*\n','',retbuf)
+
+        frame = {} # the output metric frame (dict of dict's)
+        
+        #extract the xen version
+        m = re.search('xentop.*(Xen.*?)\n',cleansed_retbuf,re.S)
+        if not m: return None
+        frame['XEN_VER'] = m.group(1)
+
+        #initialise aggregation counters
+        frame['VM_TOTAL_CPU(%)'] = 0.0
+        frame['VM_TOTAL_MEM(%)'] = 0.0
+
+        # split the returned buffer into individual frame buffers ...
+        frames = re.split('xentop.*\n',cleansed_retbuf,re.S)
+        #... and use the last frame buffer for creating the metric frame
+        fbuffer = frames[-1:][0]
+
+        # extract host cpu and mem configuration
+        m = re.search('Mem:(.*) total.*CPUs:(.*)',fbuffer)
+        if not m: return None
+        frame['SERVER_CPUs'] = m.group(2).strip()
+        frame['SERVER_MEM'] = m.group(1).strip()
+
+        # extract overall runtime domain stats
+        m = re.search('(\d+) domains.*(\d+) paused.*(\d+) crashed',fbuffer)
+        if not m: return None
+        frame['RUNNING_VMs'] = m.group(1).strip()
+        frame['PAUSED_VMs']  = m.group(2).strip()
+        frame['CRASHED_VMs'] = m.group(3).strip()
+        
+        
+        # parse the metric frame buffer for per domain stats
+        lines = fbuffer.split('\n')[:-1]
+        mbuffer = ''
+        # strip unused entries at the top of the buffer
+        # and extract the metric (sub)buffer
+        for l in lines:
+            if l.strip().startswith('NAME'):
+                mbuffer = lines[lines.index(l):]
+                break
+        # construct the metric frame as a dict of dictionaries
+        # containing metric-name, metric-value pairs
+        cleanup_exp = re.compile('[a-zA-Z]+\s[a-zA-Z]+ | n/a')
+        for d in mbuffer[1:]:
+            cleansed = re.sub(cleanup_exp,'None',d)
+            d_frame = dict(zip(mbuffer[0].split(),cleansed.split()))
+            frame[d_frame["NAME"]] = d_frame
+            # compute running aggregates
+            if d_frame['NAME'] != 'Domain-0':
+                frame['VM_TOTAL_CPU(%)'] += float(d_frame['CPU(%)'])
+                frame['VM_TOTAL_MEM(%)'] += float(d_frame['MEM(%)'])
+
+
+        return frame
+        
+    
+        
+
+        
+            
 
 
 
@@ -421,6 +519,15 @@ if __name__ == "__main__":
                            isRemote=remote)
 
     ## create/destroy dom
+    dom_config = DomConfig(managed_node, dom_file)
+    dom_config["memory"] = 256
+    dom_config["vif"] = ['bridge=xenbr1']
+    m = { 'VM_NAME':'foo', 'IMAGE_NAME':'anaconda' } 
+    dom_config.instantiate_config(m)
+    print dom_config.default_computed_options, dom_config.get_computed_options()
+    dom_config.save("/tmp/test_config")
+
+    sys.exit(0)
     
     dom_config = DomConfig(managed_node,dom_file)
     dom_name = managed_node.create_dom(dom_config)
